// ------------------------------------------------------------
// Real-time Global Illumination
// 
// Copyright (c) 2004 IMAGIRE Takashi. All rights reserved.
// ------------------------------------------------------------
#include "paraboloid.fx"

#define NV4X
 
// ------------------------------------------------------------
// Global variables(O[oϐ)
// ------------------------------------------------------------
float4x4 mWVP;		// Local-Projection matrix
float4x4 mWV;		// Local-View matrix
float4x4 mST;					// [JT[tFXWnւ̕ϊs̓]u
float4 vEye;		// Eye position(Local coordinate)

float diffuse;
float specular;
float translucent;

float MAP_WIDTH;	// texture width
float MAP_HEIGHT;	// texture height

// ------------------------------------------------------------
// Textures(eNX`)
// ------------------------------------------------------------
texture SrcTex;
sampler SrcSamp = sampler_state
{
    Texture = <SrcTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture SSTex;
sampler SSSamp = sampler_state
{
    Texture = <SSTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture DecaleTex;
sampler DecaleSamp = sampler_state
{
    Texture = <DecaleTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture PRTTex;
sampler PRTSamp = sampler_state
{
    Texture = <PRTTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
// Dual paraboloid maps(o}bv)
// ------------------------------------------------------------
texture WeightTexFront;
sampler WeightSampFront = sampler_state
{
    Texture = <WeightTexFront>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture WeightTexBack;
sampler WeightSampBack = sampler_state
{
    Texture = <WeightTexBack>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
// Dual paraboloid environment maps(o}bv)
// ------------------------------------------------------------
texture ParaboloidFrontTex;
sampler ParaboloidFrontSamp = sampler_state
{
    Texture = <ParaboloidFrontTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};
// ------------------------------------------------------------
texture ParaboloidBackTex;
sampler ParaboloidBackSamp = sampler_state
{
    Texture = <ParaboloidBackTex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;

    AddressU = Clamp;
    AddressV = Clamp;
};


// ------------------------------------------------------------
// Vertex shader output(_VF[_sNZVF[_ɓnf[^)
// ------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Pos			: POSITION;
    float4 Color		: COLOR0;
	float2 Tex0			: TEXCOORD0;
	float2 Tex1			: TEXCOORD1;
	float3 Normal		: TEXCOORD2;
	float2 DecaleTex	: TEXCOORD3;
};

// ------------------------------------------------------------
// Simple vertex shader program(wi`撸_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT VS (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Normal : NORMAL             // (@xNg)
     ,float4 Tex0   : TEXCOORD0	         // (eNX`W)
){
    VS_OUTPUT Out = (VS_OUTPUT)0;
    
	// Position(ʒuW)
	Out.Pos = mul( Pos, mWVP );
	
	// Texture coordinate(eNX`W)
    Out.Tex0 = Tex0;
	
    return Out;
}




// ------------------------------------------------------------
// Simple pixel shader program(wi`sNZVF[_vO)
// ------------------------------------------------------------
float4 PS (VS_OUTPUT In) : COLOR
{
	float4 Out;
	
	// Color(F)
	Out = tex2D( SrcSamp, In.Tex0 );
	
	return Out;
}





// ------------------------------------------------------------
// Parabploid vertex shader program(ϊ_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT VS_Parabploid (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Tex0   : TEXCOORD0	         // (eNX`W)
){
    VS_OUTPUT Out = (VS_OUTPUT)0;
    
	// Position(ʒuW)
	Out.Pos = paraboloid( Pos, mWV );
	
	// Texture coordinate(eNX`W)
    Out.Tex0 = Tex0;
	
    return Out;
}


// ------------------------------------------------------------
// Dual-Parabploid environment mapping vertex shader program(}bv_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT VS_ParabploidEnvmap (
      float4 Pos    : POSITION           // (f̒_)
    , float4 Normal : NORMAL             // (@xNg)
    , float2 Tex    : TEXCOORD0          // (eNX`W)
      )
{
    VS_OUTPUT Out = (VS_OUTPUT)0;
    float3 ray;
    float3 ref;
    
	// Position(ʒuW)
	Out.Pos = mul( Pos, mWVP );
	
	// Color(F)
	Out.Color = 0.0f;
	
	// Texture Coordinate(eNX`W)
    ray = normalize( Pos - vEye ).xyz;
	ref = reflect( ray, Normal.xyz );		// Reflection vector(˃xNg)

	Out.Tex0.x = 0.5 * ( 1 + ref.x/(1+ref.z));
	Out.Tex0.y = 0.5 * ( 1 - ref.y/(1+ref.z));
	Out.Tex1.x = 0.5 * ( 1 - ref.x/(1-ref.z));
	Out.Tex1.y = 0.5 * ( 1 - ref.y/(1-ref.z));
	
	Out.Color.w = ref.z + 0.5f;		// The judgement of the signature of the z-component of the reflection vector(˃xNg̐̔)
	
	// Normal vector in the view coordinate system(r[Wnł̖@xNg)
	Out.Normal = mul( Normal.xyz, mWV );
	
	Out.DecaleTex = Tex;
	
    return Out;
}


// ------------------------------------------------------------
// Sampling environment map(}bv̓ǂݍ)
// ------------------------------------------------------------
float4 GetEnvMap( float2 tex_front, float2 tex_back, float NE, bool front_or_back )
{
	float4 env;
	
	if( front_or_back ){
		env = tex2D( ParaboloidFrontSamp, tex_front );
	}else{
		env = tex2D( ParaboloidBackSamp,  tex_back  );
	}
	
	// Fresnel term (tl)
	float F = 1-NE*NE;
	
	return F * env;
}

// ------------------------------------------------------------
// Sampling Global Illumination(Ɩ̓ǂݍ)
// ------------------------------------------------------------
float4 GetGI( float2 tex_coord, sampler Samp )
{
#if 0
	float4 SH = 2.0 * tex2D( Samp, tex_coord ) - 1.0;
	float4 GI =   SH.a * ( 2.0 * tex2D( SrcSamp, float2(0.25, 0.25) ) - 1.0)
				+ SH.r * ( 2.0 * tex2D( SrcSamp, float2(0.25, 0.75) ) - 1.0)
				+ SH.g * ( 2.0 * tex2D( SrcSamp, float2(0.75, 0.25) ) - 1.0)
				+ SH.b * ( 2.0 * tex2D( SrcSamp, float2(0.75, 0.75) ) - 1.0);
#else
	// Tuned
	float4 SH = 4.0 * tex2D( Samp, tex_coord ) - 2.0;
	float4 GI =   SH.a * tex2D( SrcSamp, float2(0.25, 0.25))
				+ SH.r * tex2D( SrcSamp, float2(0.25, 0.75))
				+ SH.g * tex2D( SrcSamp, float2(0.75, 0.25))
				+ SH.b * tex2D( SrcSamp, float2(0.75, 0.75));
	
	GI -= dot(SH, float4(0.5,0.5,0.5,0.5));
#endif
				
	return GI;
}



// ------------------------------------------------------------
// Full Global Illumination(S݃sNZVF[_vO)
// ------------------------------------------------------------
float4 PS_ParabploidEnvmap (VS_OUTPUT In) : COLOR
{
	float4 Env = GetEnvMap( In.Tex0, In.Tex1, In.Normal.z, 0.5 < In.Color.w );// Environment map(}bv)
	float4 GI  = GetGI( In.DecaleTex, PRTSamp );						// Global Illumination
	float4 SS  = GetGI( In.DecaleTex, SSSamp );						// Sub surface
	float4 decale = tex2D( DecaleSamp, In.DecaleTex );
decale=1;
	return (diffuse * GI + translucent * SS) * decale + specular * Env;
//	return (0.3 + 3.0 * GI) * decale + Env;
}


// ------------------------------------------------------------
// No environment mapping(}bvȂsNZVF[_vO)
// ------------------------------------------------------------
float4 PS_ParabploidEnvmapNoEnv (VS_OUTPUT In) : COLOR
{
	float4 Out;
	float4 decale = tex2D( DecaleSamp, In.DecaleTex );
	float4 GI  = GetGI( In.DecaleTex, PRTSamp );						// Global Illumination
	
	// Color(F)
	Out = 3.0 * GI;// * decale;					// Global Illumination

	return Out;
}


// ------------------------------------------------------------
// No global illumination (GIȂsNZVF[_vO)
// ------------------------------------------------------------
float4 PS_ParabploidEnvmapNoGI (VS_OUTPUT In) : COLOR
{
	float4 Out;
	float4 SS  = GetGI( In.DecaleTex, SSSamp );						// Sub surface
	
	// Color(F)
	Out = 5.0 * SS;	// Sub surface

	return Out;
}





// ------------------------------------------------------------
// 64 box sampling (US{bNX TvO)
// ------------------------------------------------------------
// ------------------------------------------------------------
struct VS_OUTPUT_64BOX_SAMPLING
{
    float4 Pos			: POSITION;
	float2 Tex0			: TEXCOORD0;
	float2 Tex1			: TEXCOORD1;
	float2 Tex2			: TEXCOORD2;
	float2 Tex3			: TEXCOORD3;
	float2 Tex4			: TEXCOORD4;
	float2 Tex5			: TEXCOORD5;
	float2 Tex6			: TEXCOORD6;
	float2 Tex7			: TEXCOORD7;
};

// ------------------------------------------------------------
// Vertex shader program(_VF[_vO)
// ------------------------------------------------------------
VS_OUTPUT_64BOX_SAMPLING VS_Reduction (
      float4 Pos    : POSITION           // (f̒_)
     ,float2 Tex    : TEXCOORD0	         // (eNX`W)
){
    VS_OUTPUT_64BOX_SAMPLING Out = (VS_OUTPUT_64BOX_SAMPLING)0;
    
    // Position(ʒuW)
    Out.Pos = Pos;

	// Texture Coordinates(eNX`W)
    Out.Tex0 = Tex + float2(3.0f/MAP_WIDTH, 1.0f/MAP_HEIGHT);
    Out.Tex1 = Tex + float2(3.0f/MAP_WIDTH, 3.0f/MAP_HEIGHT);
    Out.Tex2 = Tex + float2(3.0f/MAP_WIDTH, 5.0f/MAP_HEIGHT);
    Out.Tex3 = Tex + float2(3.0f/MAP_WIDTH, 7.0f/MAP_HEIGHT);
    Out.Tex4 = Tex + float2(1.0f/MAP_WIDTH, 1.0f/MAP_HEIGHT);
    Out.Tex5 = Tex + float2(1.0f/MAP_WIDTH, 3.0f/MAP_HEIGHT);
    Out.Tex6 = Tex + float2(1.0f/MAP_WIDTH, 5.0f/MAP_HEIGHT);
    Out.Tex7 = Tex + float2(1.0f/MAP_WIDTH, 7.0f/MAP_HEIGHT);
    
    return Out;
}

// ------------------------------------------------------------
// Pixel shader program(sNZVF[_vO)
// ------------------------------------------------------------
float4 PS_Reduction ( VS_OUTPUT_64BOX_SAMPLING In ) : COLOR0
{
	float4 t0 = tex2D(SrcSamp, In.Tex0);
	float4 t1 = tex2D(SrcSamp, In.Tex1);
	float4 t2 = tex2D(SrcSamp, In.Tex2);
	float4 t3 = tex2D(SrcSamp, In.Tex3);
	
	float4 t4 = tex2D(SrcSamp, In.Tex4);
	float4 t5 = tex2D(SrcSamp, In.Tex5);
	float4 t6 = tex2D(SrcSamp, In.Tex6);
	float4 t7 = tex2D(SrcSamp, In.Tex7);
	
	float4 t8 = tex2D(SrcSamp, In.Tex0 + float2(+4.0f/MAP_WIDTH, 0));
	float4 t9 = tex2D(SrcSamp, In.Tex1 + float2(+4.0f/MAP_WIDTH, 0));
	float4 ta = tex2D(SrcSamp, In.Tex2 + float2(+4.0f/MAP_WIDTH, 0));
	float4 tb = tex2D(SrcSamp, In.Tex3 + float2(+4.0f/MAP_WIDTH, 0));
	
	float4 tc = tex2D(SrcSamp, In.Tex4 + float2(+4.0f/MAP_WIDTH, 0));
	float4 td = tex2D(SrcSamp, In.Tex5 + float2(+4.0f/MAP_WIDTH, 0));
	float4 te = tex2D(SrcSamp, In.Tex6 + float2(+4.0f/MAP_WIDTH, 0));
	float4 tf = tex2D(SrcSamp, In.Tex7 + float2(+4.0f/MAP_WIDTH, 0));
	
#ifndef NV4X
	return ((t0+t1+t2+t3)
		   +(t4+t5+t6+t7)
		   +(t8+t9+ta+tb)
		   +(tc+td+te+tf))/16;
#else //!NV4X
	return 0.25*(
			0.25*(t0+t1+t2+t3
		   +      t4+t5+t6+t7)
		   +0.25*(t8+t9+ta+tb
		   +      tc+td+te+tf));
#endif// !NV4X
}




// ------------------------------------------------------------
// SH(0,0)
// ------------------------------------------------------------
float4 PS_00 (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Tex    : TEXCOORD0	         // (eNX`W)
) : COLOR0
{
	float4 Tex2 = Tex;
	Tex2.x = 1-Tex2.x;
	
	return 0.5 + 0.25 * (
		  (2.0 * tex2D( WeightSampFront, Tex ).a - 1.0) * tex2D( ParaboloidFrontSamp, Tex ) +
		  (2.0 * tex2D( WeightSampBack,  Tex ).a - 1.0) * tex2D( ParaboloidBackSamp,  Tex2) );
}
// ------------------------------------------------------------
// SH(1,0)
// ------------------------------------------------------------
float4 PS_10 (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Tex    : TEXCOORD0	         // (eNX`W)
) : COLOR0
{
	float4 Tex2 = Tex;
	Tex2.x = 1-Tex2.x;
	
	return 0.5 + 0.25 * (
		  (2.0 * tex2D( WeightSampFront, Tex).r - 1.0) * tex2D( ParaboloidFrontSamp, Tex ) +
		  (2.0 * tex2D( WeightSampBack,  Tex).r - 1.0) * tex2D( ParaboloidBackSamp,  Tex2) );
}
// ------------------------------------------------------------
// SH(1,1)
// ------------------------------------------------------------
float4 PS_11 (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Tex    : TEXCOORD0	         // (eNX`W)
) : COLOR0
{
	float4 Tex2 = Tex;
	Tex2.x = 1-Tex2.x;
	
	return 0.5 + 0.25 * (
		  (2.0 * tex2D( WeightSampFront, Tex).g - 1.0) * tex2D( ParaboloidFrontSamp, Tex ) +
		  (2.0 * tex2D( WeightSampBack,  Tex).g - 1.0) * tex2D( ParaboloidBackSamp,  Tex2) );
}
// ------------------------------------------------------------
// SH(1,2)
// ------------------------------------------------------------
float4 PS_12 (
      float4 Pos    : POSITION           // (f̒_)
     ,float4 Tex    : TEXCOORD0	         // (eNX`W)
) : COLOR0
{
	float4 Tex2 = Tex;
	Tex2.x = 1-Tex2.x;
	
	return 0.5 + 1.7 * (
//	return 0.5 + 0.25 * (//???
		  (2.0 * tex2D( WeightSampFront, Tex).b - 1.0) * tex2D( ParaboloidFrontSamp, Tex ) +
		  (2.0 * tex2D( WeightSampBack,  Tex).b - 1.0) * tex2D( ParaboloidBackSamp,  Tex2) );
}


// ------------------------------------------------------------
// ------------------------------------------------------------
// Create height map and normal map(ʒuA@}bv̍쐬)
// ------------------------------------------------------------
// ------------------------------------------------------------
struct VS_OUTPUT_MAP
{
    float4 Pos			: POSITION;
	float4 Position		: TEXCOORD0;
	float4 Normal		: TEXCOORD1;
};
struct PS_OUTPUT_MAP
{
    float4 Position		: COLOR0;
    float4 Normal		: COLOR1;
};

// ------------------------------------------------------------
// Vertex shader(_VF[_)
// ------------------------------------------------------------
VS_OUTPUT_MAP VS_Map (
      float4 Pos    : POSITION           // f̒_
    , float4 Normal : NORMAL             // @xNg
    , float4 Tex0   : TEXCOORD0	         // eNX`W
){
    float MAP_SIZE = 512;
    VS_OUTPUT_MAP Out = (VS_OUTPUT_MAP)0;        // o̓f[^
    
    float4 pos = mul( Pos, mWVP );
    
	// ʒuW
	Out.Pos.x =  2.0 * (Tex0.x*(MAP_SIZE+1)/MAP_SIZE - 1/MAP_SIZE) - 1.0;
	Out.Pos.y = -2.0 * (Tex0.y*(MAP_SIZE+1)/MAP_SIZE - 1/MAP_SIZE) + 1.0;
	Out.Pos.z = 0.5;
	Out.Pos.w = 1;
	
	// F
	Out.Position = Pos;
	Out.Normal   = Normal;
	Out.Normal.w = 0;

    return Out;
}


// ------------------------------------------------------------
// Pixel shader(sNZVF[_)
// ------------------------------------------------------------
PS_OUTPUT_MAP PS_Map (VS_OUTPUT_MAP In)
{
	PS_OUTPUT_MAP Out;
	
	Out.Position = In.Position;
	Out.Normal   = normalize(In.Normal);

	return Out;
}



// ------------------------------------------------------------
// ------------------------------------------------------------
// PRT
// ------------------------------------------------------------
// ------------------------------------------------------------
struct VS_OUTPUT_SH
{
    float4 Pos			: POSITION;
    float4 Tex			: TEXCOORD0;
    float4 Position		: TEXCOORD1;
};


// ------------------------------------------------------------
// Vertex shader(_VF[_)
// ------------------------------------------------------------
VS_OUTPUT_SH VS_SH (
      float4 Pos    : POSITION           // f̒_
    , float4 Tex0   : TEXCOORD0	         // eNX`W
){
    VS_OUTPUT_SH Out = (VS_OUTPUT_SH)0;        // o̓f[^
    
	// ʒuW
	Out.Pos = Pos;
	
    // ʒuxNg
    Out.Position = Pos;
    Out.Position.zw = 0;
    
	// eNX`W
	Out.Tex   = Tex0;

    return Out;
}


// ------------------------------------------------------------
// SH(0,0), SH(1,-1), SH(1,0), SH(1,1)
// ------------------------------------------------------------
float4 PS_SH0 (VS_OUTPUT_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.w = 1;
	val.x = -N.x;
	val.y =  N.y;
	val.z = -N.z;

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ));
}




// ------------------------------------------------------------
// SH(2,-2), SH(2,-1), SH(2,0), SH(2,1)
// ------------------------------------------------------------
float4 PS_SH1 (VS_OUTPUT_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.x = N.x * N.x - N.z * N.z;
	val.y = - N.y * N.x;
	val.z =  (N.y * N.y - 1/3)/2;
	val.w = - N.y * N.z;

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ));
}


// ------------------------------------------------------------
// SH(2,2), SH(3,-3), SH(3,-2), SH(3,-1)
// ------------------------------------------------------------
float4 PS_SH2 (VS_OUTPUT_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.x = 2 * N.x * N.z;
	val.y = - N.z * (N.x * N.x - N.z * N.z/3);
	val.z = 2 * N.x * N.y * N.z;
	val.w = 0.5 * N.z * (0.2 - N.y * N.y);

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ));
}
// ------------------------------------------------------------
// SH(3,0), SH(3,1), SH(3,2), SH(3,3)
// ------------------------------------------------------------
float4 PS_SH3 (VS_OUTPUT_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.x = N.y * (N.y * N.y - 3/5) / 6;
	val.y = 0.5 * N.x * (0.2 - N.y * N.y);
	val.z = N.y * (N.x * N.x - N.z * N.z);
	val.w = -N.x * (N.x * N.x/3 - N.z * N.z);

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ));
}


// ------------------------------------------------------------
// ------------------------------------------------------------
// Օ̌vZ
// ------------------------------------------------------------
// ------------------------------------------------------------
struct VS_OUTPUT_RADIANCE
{
    float4 Pos			: POSITION;
    float4 Color		: COLOR0;
};

// ------------------------------------------------------------
// _VF[_vO
// ------------------------------------------------------------
VS_OUTPUT_RADIANCE VS_Radiance (
      float4 Pos    : POSITION           // f̒_
     ,float4 Normal : NORMAL             // @xNg
     ,float4 Tex0   : TEXCOORD0	         // eNX`W
){
    VS_OUTPUT_RADIANCE Out = (VS_OUTPUT_RADIANCE)0;        // o̓f[^
    
    float4 pos = mul( Pos, mWV );
    
	// ʒuW
	float rlen = rsqrt(dot(pos.xyz, pos.xyz));
	pos *= rlen;
	Out.Pos = pos;
	// wʂɗ_͖ɈL΂
	if(pos.z<=0){
		Out.Pos.w = -0.00001;
	}else{
		Out.Pos.w = 1;
	}
	
    Out.Color = 0.5;
    
    return Out;
}
// ------------------------------------------------------------
// sNZVF[_vO
// ------------------------------------------------------------
float4 PS_Radiance (VS_OUTPUT_RADIANCE In) : COLOR
{
	return In.Color;
}

// ------------------------------------------------------------
// ------------------------------------------------------------
// Օ̌vZ
// ------------------------------------------------------------
// ------------------------------------------------------------
struct VS_OUTPUT_SS_RADIANCE
{
    float4 Pos			: POSITION;
    float4 Color		: COLOR0;
	float4 ProjPos      : TEXCOORD0;
};

struct PS_OUTPUT_SS_RADIANCE
{
    float4 Color		: COLOR0;
    float  Depth		: DEPTH;
};

// ------------------------------------------------------------
// _VF[_vO
// ------------------------------------------------------------
VS_OUTPUT_SS_RADIANCE VS_SSRadiance (
      float4 Pos    : POSITION           // f̒_
     ,float4 Normal : NORMAL             // @xNg
     ,float4 Tex0   : TEXCOORD0	         // eNX`W
){
    VS_OUTPUT_SS_RADIANCE Out = (VS_OUTPUT_SS_RADIANCE)0;        // o̓f[^
    
    float4 pos = mul( Pos, mWV );
    
	// ʒuW
	float rlen = rsqrt(dot(pos.xyz, pos.xyz));
	pos *= rlen;
	// wʂɗ_͖ɈL΂
	if(pos.z<=0){
		pos.w = -0.00001;
	}else{
		pos.w = 1;
	}
	Out.Pos = pos;
	Out.ProjPos = pos;
	
	float color = exp( -0.5f * pos.z / pos.w );
    Out.Color = color;
    
    return Out;
}
// ------------------------------------------------------------
// sNZVF[_vO
// ------------------------------------------------------------
PS_OUTPUT_SS_RADIANCE PS_SSRadiance (VS_OUTPUT_SS_RADIANCE In)
{
	PS_OUTPUT_SS_RADIANCE o;
	
	// ʂƂ͋t̂r
	o.Depth = 1.0f-In.ProjPos.z/In.ProjPos.w;
	
	o.Color = In.Color;

	return o;
}



// ------------------------------------------------------------
// ------------------------------------------------------------
// SSPRT
// ------------------------------------------------------------
// ------------------------------------------------------------
struct VS_OUTPUT_SS_SH
{
    float4 Pos			: POSITION;
    float4 Tex			: TEXCOORD0;
    float2 Tex2 		: TEXCOORD1;
    float4 Position		: TEXCOORD2;
};


// ------------------------------------------------------------
// Vertex shader(_VF[_)
// ------------------------------------------------------------
VS_OUTPUT_SS_SH VS_SS_SH (
      float4 Pos    : POSITION           // f̒_
    , float4 Tex0   : TEXCOORD0	         // eNX`W
){
    VS_OUTPUT_SS_SH Out = (VS_OUTPUT_SS_SH)0;        // o̓f[^
    
	// ʒuW
	Out.Pos = Pos;
	
    // ʒuxNg
    Out.Position = Pos;
    Out.Position.zw = 0;
    
	// eNX`W
	Out.Tex   = Tex0;
	Out.Tex2.x= Tex0.x;
	Out.Tex2.y= 1-Tex0.y;

    return Out;
}


// ------------------------------------------------------------
// SH(0,0), SH(1,-1), SH(1,0), SH(1,1)
// ------------------------------------------------------------
float4 PS_SS_SH0 (VS_OUTPUT_SS_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.w = 1;
	val.x = -N.x;
	val.y =  N.y;
	val.z = -N.z;

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ) * tex2D( SSSamp, In.Tex2 ));
}




// ------------------------------------------------------------
// SH(2,-2), SH(2,-1), SH(2,0), SH(2,1)
// ------------------------------------------------------------
float4 PS_SS_SH1 (VS_OUTPUT_SS_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.x = N.x * N.x - N.z * N.z;
	val.y = - N.y * N.x;
	val.z =  (N.y * N.y - 1/3)/2;
	val.w = - N.y * N.z;

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ) * tex2D( SSSamp, In.Tex2 ));
}


// ------------------------------------------------------------
// SH(2,2), SH(3,-3), SH(3,-2), SH(3,-1)
// ------------------------------------------------------------
float4 PS_SS_SH2 (VS_OUTPUT_SS_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.x = 2 * N.x * N.z;
	val.y = - N.z * (N.x * N.x - N.z * N.z/3);
	val.z = 2 * N.x * N.y * N.z;
	val.w = 0.5 * N.z * (0.2 - N.y * N.y);

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ) * tex2D( SSSamp, In.Tex2 ));
}
// ------------------------------------------------------------
// SH(3,0), SH(3,1), SH(3,2), SH(3,3)
// ------------------------------------------------------------
float4 PS_SS_SH3 (VS_OUTPUT_SS_SH In) : COLOR0
{
	float4 N = In.Position;
	float4 val;
	
	float l = dot(N,N);
	if(1.0<l) N *= rsqrt(l);
	N.z = sqrt(1-dot(N,N));
	// T[tFXWn烍[JWnɕϊ
	N = mul( N, mST );
	
	val.x = N.y * (N.y * N.y - 3/5) / 6;
	val.y = 0.5 * N.x * (0.2 - N.y * N.y);
	val.z = N.y * (N.x * N.x - N.z * N.z);
	val.w = -N.x * (N.x * N.x/3 - N.z * N.z);

	return 0.5+0.5 * (val * tex2D( SrcSamp, In.Tex ) * tex2D( SSSamp, In.Tex2 ));
}






// ------------------------------------------------------------
// eNjbN
// ------------------------------------------------------------
technique TShader
{
    pass P0	// Full Global Illuminations
    {
        VertexShader = compile vs_1_1 VS_ParabploidEnvmap();
        PixelShader  = compile ps_2_0 PS_ParabploidEnvmap();
    }
    pass P1	// No environment mapping(}bvȂ)
    {
        VertexShader = compile vs_1_1 VS_ParabploidEnvmap();
        PixelShader  = compile ps_2_0 PS_ParabploidEnvmapNoEnv();
    }
    pass P2	// No Global illumination(GIȂ)
    {
        VertexShader = compile vs_1_1 VS_ParabploidEnvmap();
        PixelShader  = compile ps_2_0 PS_ParabploidEnvmapNoGI();
    }
    
    pass P3	// reduction(k)
    {
        VertexShader = compile vs_1_1 VS_Reduction();
        PixelShader  = compile ps_2_0 PS_Reduction();
    }
    
    pass P4	// Paraboloid transform(ϊn``)
    {
        VertexShader = compile vs_1_1 VS_Parabploid();
        PixelShader  = compile ps_1_1 PS();
    }
    pass P5 // Simple transform(n``)
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_1 PS();
    }
    
    pass P6
    {
        PixelShader = compile ps_2_0 PS_00();
    }
    pass P7
    {
        PixelShader = compile ps_2_0 PS_10();
    }
    pass P8
    {
        PixelShader = compile ps_2_0 PS_11();
    }
    pass P9
    {
        PixelShader = compile ps_2_0 PS_12();
    }    
    pass P10 // Height map and normal map(ʒuA@}bv)
    {
        VertexShader = compile vs_1_1 VS_Map();
        PixelShader  = compile ps_2_0 PS_Map();
    }
    pass P11
    {
        VertexShader = compile vs_1_1 VS_SH();
        PixelShader  = compile ps_2_0 PS_SH0();
    }
    pass P12
    {
        VertexShader = compile vs_1_1 VS_SH();
        PixelShader  = compile ps_2_0 PS_SH1();
    }
    pass P13
    {
        VertexShader = compile vs_1_1 VS_SH();
        PixelShader  = compile ps_2_0 PS_SH2();
    }
    pass P14
    {
        VertexShader = compile vs_1_1 VS_SH();
        PixelShader  = compile ps_2_0 PS_SH3();
    }
    pass P15 // fBAX̌vZ
    {
        VertexShader = compile vs_1_1 VS_Radiance();
        PixelShader  = compile ps_2_0 PS_Radiance();
    }
    pass P16 // SSpfBAX̌vZ
    {
        VertexShader = compile vs_2_0 VS_SSRadiance();
        PixelShader  = compile ps_2_0 PS_SSRadiance();
		cullmode = none;
    }
    pass P17
    {
        VertexShader = compile vs_1_1 VS_SS_SH();
        PixelShader  = compile ps_2_0 PS_SS_SH0();
    }
    pass P18
    {
        VertexShader = compile vs_1_1 VS_SS_SH();
        PixelShader  = compile ps_2_0 PS_SS_SH1();
    }
    pass P19
    {
        VertexShader = compile vs_1_1 VS_SS_SH();
        PixelShader  = compile ps_2_0 PS_SS_SH2();
    }
    pass P20
    {
        VertexShader = compile vs_1_1 VS_SS_SH();
        PixelShader  = compile ps_2_0 PS_SS_SH3();
    }
}
